export class Main extends Indicator {

    constructor() {
        super();

        this.setProperty(Property.SeparateWindow, true);
        this.setProperty(Property.Minimum, 0);
        this.setProperty(Property.Maximum, 100);

        this.addLevel(40);
        this.addLevel(60);

        this.addInput("RBVIPeriod", InputType.int, 10, {
            min: 2
        });
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffer = this.addBuffer();

        this.arr = {
            PosBuffer : [],
            NegBuffer : [],
        };
    }

    onInit() {
        let {RBVIPeriod} = this.getInputs();

        this.buffer
            .setShape(Shape.Line)
            .setColor(Color.RoyalBlue)
            .setLabel("RBVI")
            .setDrawBegin(RBVIPeriod)
            .setShortName("RBVI(" + RBVIPeriod + ")");
    }

    async onUpdate() {
        const {PosBuffer, NegBuffer} = this.arr,
            {RBVIPeriod, PriceType} = this.getInputs(),
            {Volume} = await Bar.load([Bar.Mode.Volume], PriceType),
            barSize = Volume.length;
        ;

        let i, countedBars = Bar.counted(),
            rel, negative, positive, sumn, sump, listATR, v;

        if (barSize <= RBVIPeriod) return;
        if (countedBars < 1) {
            for (i = 1; i <= RBVIPeriod; i++) {
                this.buffer.set(barSize - i, 50);
            }
        }
        i = barSize - RBVIPeriod - 1;
        if (countedBars >= RBVIPeriod) {
            i = barSize - countedBars - 1;
        }
        listATR = await Indicators.ATR(Current.Symbol, Current.TimeFrame, RBVIPeriod, PriceType);
        while (i >= 0) {
            sumn = 0;
            sump = 0;
            rel = listATR[i] * Volume[i] - listATR[i + 1] * Volume[i+1];

            if (rel > 0) {
                sump = rel;
            } else {
                sumn = -rel;
            }
            v = PosBuffer[i + 1] || 0;
            positive = (v * (RBVIPeriod - 1) + sump) / RBVIPeriod;
            v = NegBuffer[i + 1] || 0;
            negative = (v * (RBVIPeriod - 1) + sumn) / RBVIPeriod;
            PosBuffer[i] = positive;
            NegBuffer[i] = negative;
            if (negative === 0) {
                this.buffer.set(i, 0);
            } else {
                this.buffer.set(i, 100 * (1 - 1 / (1 + positive / negative)));
            }
            i--;
        }
    }
}

